@@ -152,6 +152,34 @@ static const char *type2string(enum connman_device_type type)
 	return NULL;
 }
 
+enum connman_service_type __connman_device_get_service_type(struct connman_device *device)
+{
+	enum connman_device_type type = connman_device_get_type(device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		break;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+		return CONNMAN_SERVICE_TYPE_ETHERNET;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+		return CONNMAN_SERVICE_TYPE_WIFI;
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		return CONNMAN_SERVICE_TYPE_WIMAX;
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+		return CONNMAN_SERVICE_TYPE_CELLULAR;
+	}
+
+	return CONNMAN_SERVICE_TYPE_UNKNOWN;
+}
+
 static int set_connected(struct connman_device *device,
 						connman_bool_t connected)
 {
@@ -202,6 +230,18 @@ static int set_connected(struct connman_device *device,
 					CONNMAN_SERVICE_STATE_IDLE);
 	}
 
+	if (connected == TRUE) {
+		enum connman_service_type type;
+
+		type = __connman_device_get_service_type(device);
+		__connman_notifier_connect(type);
+	} else {
+		enum connman_service_type type;
+
+		type = __connman_device_get_service_type(device);
+		__connman_notifier_disconnect(type);
+	}
+
 	return 0;
 }
 
@@ -249,6 +289,7 @@ static int powered_changed(struct connman_device *device)
 static int set_powered(struct connman_device *device, connman_bool_t powered)
 {
 	struct connman_device_driver *driver = device->driver;
+	enum connman_service_type type;
 	int err;
 
 	DBG("device %p powered %d", device, powered);
@@ -259,10 +300,13 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	if (!driver)
 		return -EINVAL;
 
+	type = __connman_device_get_service_type(device);
+
 	if (powered == TRUE) {
 		if (driver->enable) {
 			err = driver->enable(device);
-			__connman_notifier_device_type_increase(device->type);
+			if (err == 0)
+				__connman_notifier_enable(type);
 		} else
 			err = -EINVAL;
 	} else {
@@ -272,7 +316,8 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 		if (driver->disable) {
 			err = driver->disable(device);
-			__connman_notifier_device_type_decrease(device->type);
+			if (err == 0)
+				__connman_notifier_disable(type);
 		} else
 			err = -EINVAL;
 	}
@@ -826,9 +871,14 @@ static void device_enable(struct connman_device *device)
 		return;
 
 	if (device->driver->enable) {
-		if (device->driver->enable(device) == 0)
+		if (device->driver->enable(device) == 0) {
+			enum connman_service_type type;
+
 			device->powered = TRUE;
-		__connman_notifier_device_type_increase(device->type);
+
+			type = __connman_device_get_service_type(device);
+			__connman_notifier_enable(type);
+		}
 	}
 }
 
@@ -842,9 +892,14 @@ static void device_disable(struct connman_device *device)
 	g_hash_table_remove_all(device->networks);
 
 	if (device->driver->disable) {
-		if (device->driver->disable(device) == 0)
+		if (device->driver->disable(device) == 0) {
+			enum connman_service_type type;
+
 			device->powered = FALSE;
-		__connman_notifier_device_type_decrease(device->type);
+
+			type = __connman_device_get_service_type(device);
+			__connman_notifier_disable(type);
+		}
 	}
 }
 
@@ -1314,6 +1369,8 @@ connman_bool_t connman_device_get_secondary(struct connman_device *device)
 int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered)
 {
+	enum connman_service_type type;
+
 	DBG("driver %p powered %d", device, powered);
 
 	if (device->timeout > 0) {
@@ -1334,6 +1391,13 @@ int connman_device_set_powered(struct connman_device *device,
 
 	device->powered = powered;
 
+	type = __connman_device_get_service_type(device);
+
+	if (device->powered == TRUE)
+		__connman_notifier_enable(type);
+	else
+		__connman_notifier_disable(type);
+
 	if (device->registered == FALSE)
 		return 0;
 
@@ -1694,12 +1758,22 @@ int __connman_device_set_offlinemode(connman_bool_t offlinemode)
 
 void __connman_device_increase_connections(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	device->connections++;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_connect(type);
 }
 
 void __connman_device_decrease_connections(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	device->connections--;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_disconnect(type);
 }
 
 /**
@@ -1809,6 +1883,8 @@ void __connman_device_set_network(struct connman_device *device,
  */
 int connman_device_register(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	__connman_storage_load_device(device);
 
 	switch (device->mode) {
@@ -1821,7 +1897,8 @@ int connman_device_register(struct connman_device *device)
 		break;
 	}
 
-	__connman_notifier_device_type_register(device->type);
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_register(type);
 
 	return connman_element_register(&device->element, NULL);
 }
@@ -1834,9 +1911,12 @@ int connman_device_register(struct connman_device *device)
  */
 void connman_device_unregister(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	__connman_storage_save_device(device);
 
-	__connman_notifier_device_type_unregister(device->type);
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_unregister(type);
 
 	connman_element_unregister(&device->element);
 }
