@@ -210,6 +210,26 @@ static enum connman_service_error string2error(const char *error)
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
 }
 
+const char *__connman_service_default(void)
+{
+	struct connman_service *service;
+	GSequenceIter *iter;
+
+	iter = g_sequence_get_begin_iter(service_list);
+
+	if (g_sequence_iter_is_end(iter) == TRUE)
+		return "";
+
+	service = g_sequence_get(iter);
+	if (service == NULL)
+		return "";
+
+	if (service->state != CONNMAN_SERVICE_STATE_READY)
+		return "";
+
+	return type2string(service->type);
+}
+
 static void state_changed(struct connman_service *service)
 {
 	DBusMessage *signal;
@@ -794,11 +814,6 @@ void __connman_service_put(struct connman_service *service)
 		iter = g_hash_table_lookup(service_hash, service->identifier);
 		if (iter != NULL) {
 			__connman_service_disconnect(service);
-
-			service->state = CONNMAN_SERVICE_STATE_FAILURE;
-			service->error = CONNMAN_SERVICE_ERROR_OUT_OF_RANGE;
-			state_changed(service);
-
 			g_sequence_remove(iter);
 		} else
 			service_free(service);
@@ -968,6 +983,30 @@ int __connman_service_set_carrier(struct connman_service *service,
 	return connman_service_set_favorite(service, carrier);
 }
 
+static void default_changed(void)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "DefaultTechnology";
+	const char *str = __connman_service_default();
+
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_STRING_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING, &str);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+}
+
 int __connman_service_indicate_state(struct connman_service *service,
 					enum connman_service_state state)
 {
@@ -984,6 +1023,9 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (service->state == state)
 		return -EALREADY;
 
+	if (service->state == CONNMAN_SERVICE_STATE_READY)
+		default_changed();
+
 	if (service->state == CONNMAN_SERVICE_STATE_FAILURE &&
 				state == CONNMAN_SERVICE_STATE_IDLE)
 		return -EINVAL;
@@ -1051,6 +1093,8 @@ int __connman_service_indicate_default(struct connman_service *service)
 {
 	DBG("service %p", service);
 
+	default_changed();
+
 	return 0;
 }
 
@@ -1413,38 +1457,12 @@ struct connman_service *__connman_service_lookup_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
 }
 
-static enum connman_service_type convert_device_type(struct connman_device *device)
-{
-	enum connman_device_type type = connman_device_get_type(device);
-
-	switch (type) {
-	case CONNMAN_DEVICE_TYPE_UNKNOWN:
-	case CONNMAN_DEVICE_TYPE_VENDOR:
-	case CONNMAN_DEVICE_TYPE_WIFI:
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_MBM:
-	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
-		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
-		return CONNMAN_SERVICE_TYPE_ETHERNET;
-	}
-
-	return CONNMAN_SERVICE_TYPE_UNKNOWN;
-}
-
 /**
  * __connman_service_create_from_device:
  * @device: device structure
@@ -1463,29 +1481,28 @@ struct connman_service *__connman_service_create_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
 
 	if (service->path != NULL) {
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		__connman_profile_changed(TRUE);
+		return service;
 	}
 
-	service->type = convert_device_type(device);
+	service->type = __connman_device_get_service_type(device);
 
 	service->device = device;
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
@@ -1522,9 +1539,7 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
@@ -1663,23 +1678,6 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	const char *ident, *group;
 	char *name;
 
-	service = __connman_service_lookup_from_network(network);
-	if (service != NULL) {
-		if (g_atomic_int_get(&service->refcount) == 0) {
-			if (service->timeout > 0) {
-				g_source_remove(service->timeout);
-				service->timeout = 0;
-			}
-
-			set_idle(service);
-		}
-
-		connman_service_ref(service);
-
-		update_from_network(service, network);
-		return service;
-	}
-
 	ident = __connman_network_get_ident(network);
 	if (ident == NULL)
 		return NULL;
@@ -1688,24 +1686,21 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	if (group == NULL)
 		return NULL;
 
-	if (__connman_network_get_weakness(network) == TRUE)
-		return NULL;
-
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
+
+	if (__connman_network_get_weakness(network) == TRUE)
+		return service;
 
 	if (service->path != NULL) {
 		update_from_network(service, network);
-
 		__connman_profile_changed(TRUE);
-
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		return service;
 	}
 
 	service->type = convert_network_type(network);
@@ -1716,12 +1711,11 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
